@@ -1,10 +1,11 @@
-from typing import Tuple, List, Callable, Any
-
+from typing import Tuple, List, Callable, Any, Dict
+import torch
 import numpy as np
 from docarray import Document, DocumentArray
 from docarray.math.distance.numpy import cosine
 
-from clip_server.model import clip
+
+from clip_server.model.tokenization import Tokenizer
 
 
 def numpy_softmax(x: 'np.ndarray', axis: int = -1) -> 'np.ndarray':
@@ -20,49 +21,65 @@ def preproc_image(
     preprocess_fn: Callable,
     device: str = 'cpu',
     return_np: bool = False,
-) -> Tuple['DocumentArray', List[Any]]:
-    contents = da.contents
+) -> Tuple['DocumentArray', Dict]:
+
+    tensors_batch = []
 
     for d in da:
+        content = d.content
+
         if d.blob:
             d.convert_blob_to_image_tensor()
         elif d.tensor is None and d.uri:
             # in case user uses HTTP protocol and send data via curl not using .blob (base64), but in .uri
             d.load_uri_to_image_tensor()
 
-        d.tensor = preprocess_fn(d.tensor).detach()
+        tensors_batch.append(preprocess_fn(d.tensor).detach())
+
+        # recover doc content
+        if d.tags.pop('__loaded_by_CAS__', False):
+            d.pop('tensor')
+        else:
+            d.content = content
+
+    tensors_batch = torch.stack(tensors_batch).type(torch.float32)
 
     if return_np:
-        da.tensors = da.tensors.cpu().numpy().astype(np.float32)
+        tensors_batch = tensors_batch.cpu().numpy()
     else:
-        da.tensors = da.tensors.to(device)
+        tensors_batch = tensors_batch.to(device)
 
-    return da, contents
+    return da, {'pixel_values': tensors_batch}
 
 
 def preproc_text(
-    da: 'DocumentArray', device: str = 'cpu', return_np: bool = False
-) -> Tuple['DocumentArray', List[Any]]:
-    contents = da.contents
+    da: 'DocumentArray',
+    tokenizer: 'Tokenizer',
+    device: str = 'cpu',
+    return_np: bool = False,
+) -> Tuple['DocumentArray', Dict]:
 
-    da.tensors = clip.tokenize(contents).detach()
+    inputs = tokenizer(da.texts)
+    inputs['input_ids'] = inputs['input_ids'].detach()
 
     if return_np:
-        da.tensors = da.tensors.cpu().numpy().astype(np.int64)
+        inputs['input_ids'] = inputs['input_ids'].cpu().numpy().astype(np.int32)
+        inputs['attention_mask'] = (
+            inputs['attention_mask'].cpu().numpy().astype(np.int32)
+        )
     else:
-        da.tensors = da.tensors.to(device)
+        inputs['input_ids'] = inputs['input_ids'].to(device)
+        inputs['attention_mask'] = inputs['attention_mask'].to(device)
 
     da[:, 'mime_type'] = 'text'
-    return da, contents
+    return da, inputs
 
 
 def split_img_txt_da(doc: 'Document', img_da: 'DocumentArray', txt_da: 'DocumentArray'):
-    if doc.uri:
-        img_da.append(doc)
-    elif doc.blob or (doc.tensor is not None):
-        img_da.append(doc)
-    elif doc.text:
+    if doc.text:
         txt_da.append(doc)
+    elif doc.blob or (doc.tensor is not None) or doc.uri:
+        img_da.append(doc)
 
 
 def set_rank(docs, _logit_scale=np.exp(4.60517)):
@@ -101,3 +118,9 @@ def set_rank(docs, _logit_scale=np.exp(4.60517)):
         )
 
         q.matches = final
+
+
+def get_image_size(name: str):
+    from clip_server.model.pretrained_models import _VISUAL_MODEL_IMAGE_SIZE
+
+    return _VISUAL_MODEL_IMAGE_SIZE[name]
